package com.sleepycat.je.recovery;

import com.sleepycat.je.CheckpointConfig;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.config.LongConfigParam;
import com.sleepycat.je.dbi.DbConfigManager;
import com.sleepycat.je.utilint.DbLsn;
import com.sleepycat.je.utilint.PropUtil;

public privileged aspect CustomizeCheckpointerTimeInterval {
	public static final LongConfigParam EnvironmentParams.CHECKPOINTER_WAKEUP_INTERVAL = new LongConfigParam(
			"je.checkpointer.wakeupInterval",
			new Long(1000000), // min
			new Long(4294967296L), // max
			new Long(0), // default
			false, // mutable
			"# The checkpointer wakeup interval in microseconds. By default, this\n"
					+ "# is inactive and we wakeup the checkpointer as a function of the\n"
					+ "# number of bytes written to the log. (je.checkpointer.bytesInterval)");

	/**
	 * set a wakeup period for the daemon thread.
	 */
	long around(Checkpointer cp) throws DatabaseException:
		execution(long CheckpointerDaemon.hook_getInterval(Checkpointer)) && args(cp){

		return getWakeupPeriod(cp.getEnv().getConfigManager());
	}

	/**
	 * Figure out the wakeup period. Supplied through this static method because
	 * we need to pass wakeup period to the superclass and need to do the
	 * calcuation outside this constructor.
	 */
	public static long getWakeupPeriod(DbConfigManager configManager)
			throws IllegalArgumentException, DatabaseException {

		long wakeupPeriod = PropUtil.microsToMillis(configManager
				.getLong(EnvironmentParams.CHECKPOINTER_WAKEUP_INTERVAL));
		long bytePeriod = configManager
				.getLong(EnvironmentParams.CHECKPOINTER_BYTES_INTERVAL);

		/* Checkpointing period must be set either by time or by log size. */
		if ((wakeupPeriod == 0) && (bytePeriod == 0)) {
			throw new IllegalArgumentException(
					EnvironmentParams.CHECKPOINTER_BYTES_INTERVAL.getName()
							+ " and "
							+ EnvironmentParams.CHECKPOINTER_WAKEUP_INTERVAL
									.getName() + " cannot both be 0. ");
		}

		/*
		 * Checkpointing by log size takes precendence over time based period.
		 */
		if (bytePeriod == 0) {
			return wakeupPeriod;
		} else {
			return 0;
		}
	}

	/**
	 * additional check for time.
	 */
	boolean around(CheckpointConfig config, Checkpointer cp) throws DatabaseException:
		execution(boolean Checkpointer.isRunnable(CheckpointConfig)) && args(config) && this(cp){

		boolean runnable = proceed(config, cp);

		if (!runnable) {
			long useTimeInterval = 0;
			if (config.getMinutes() != 0) {
				// convert to millis
				useTimeInterval = config.getMinutes() * 60 * 1000;
			} else {
				useTimeInterval = getWakeupPeriod(cp.getEnv()
						.getConfigManager());
			}

			if (useTimeInterval != 0) {
				/*
				 * Our checkpoint is determined by time. If enough time has
				 * passed and some log data has been written, do a checkpoint.
				 */
				long lastUsedLsn = cp.getEnv().getFileManager()
						.getLastUsedLsn();
				if (((System.currentTimeMillis() - cp.lastCheckpointMillis) >= useTimeInterval)
						&& (DbLsn.compareTo(lastUsedLsn, cp.lastCheckpointEnd) != 0)) {
					return true;
				}
			}
		}

		return runnable;

	}
	
	
	private int CheckpointConfig.minutes = 0;
	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public void CheckpointConfig.setMinutes(int minutes) {
		this.minutes = minutes;
	}

	/**
	 * Javadoc for this public method is generated via the doc templates in the
	 * doc_src directory.
	 */
	public int CheckpointConfig.getMinutes() {
		return minutes;
	}
}
